/*
 * Copyright 2002,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.commons.jexl.parser;

import org.apache.commons.jexl.util.Coercion;
import org.apache.commons.jexl.JexlContext;

/**
 *  Subtraction
 *
 *  @author <a href="mailto:geirm@apache.org">Geir Magnusson Jr.</a>
 *  @author <a href="mailto:mhw@kremvax.net">Mark H. Wilkinson</a>
 *  @version $Id: ASTSubtractNode.java,v 1.5 2004/02/28 13:45:20 yoavs Exp $
 */
public class ASTSubtractNode extends SimpleNode
{
    public ASTSubtractNode(int id)
    {
        super(id);
    }

    public ASTSubtractNode(Parser p, int id)
    {
        super(p, id);
    }

    public Object value(JexlContext context)
        throws Exception
    {
        Object left = ((SimpleNode) jjtGetChild(0)).value(context);
        Object right = ((SimpleNode) jjtGetChild(1)).value(context);

        /*
         *  the spec says 'and', I think 'or'
         */
        if (left == null && right == null)
            return new Integer(0);

        /*
         *  if anything is float, double or string with ( "." | "E" | "e")
         *  coerce all to doubles and do it
         */
        if ( left instanceof Float || left instanceof Double
            || right instanceof Float || right instanceof Double
            || (  left instanceof String
                 && (  ((String) left).indexOf(".") != -1 ||
                    ((String) left).indexOf("e") != -1 ||
                    ((String) left).indexOf("E") != -1 )
            )
            || (  right instanceof String
                && (  ((String) right).indexOf(".") != -1 ||
                ((String) right).indexOf("e") != -1 ||
                ((String) right).indexOf("E") != -1 )
               )
        )
        {
            Double l = Coercion.coerceDouble(left);
            Double r = Coercion.coerceDouble(right);

            return new Double(l.doubleValue() - r.doubleValue());
        }

        /*
         * otherwise to longs with thee!
         */

        Long l = Coercion.coerceLong(left);
        Long r = Coercion.coerceLong(right);

        /*
         *  TODO - this is actually wrong - JSTL says to return a Long
         *  but we have problems where you have something like 
         * 
         *     foo.bar( a - b )
         * 
         *  where bar wants an int... 
         * 
         */
        long v = l.longValue() - r.longValue();

        if ( left instanceof Integer && right instanceof Integer )
        {
            return new Integer((int) v);
        }
        else
        {
            return new Long(v);
        }
     }
    /** Accept the visitor. **/
    public Object jjtAccept(ParserVisitor visitor, Object data)
    {
        return visitor.visit(this, data);
    }
}
